﻿//=======================================================================
// Copyright Martin "quill18" Glaude 2015.
//		http://quill18.com
//=======================================================================

using System;
using System.Linq;
using UnityEngine;
using System.Collections.Generic;
using UnityEngine.SceneManagement;
using System.Xml.Serialization;
using System.IO;

public class WorldController : MonoBehaviour {

	public static WorldController Instance { get; protected set; }

	// The world and tile data
	public World world { get; protected set; }

	static bool loadWorld = false;

	// Use this for initialization
	void OnEnable () {
		if(Instance != null) {
			Debug.LogError("There should never be two world controllers.");
		}
		Instance = this;

		if(loadWorld) {
			loadWorld = false;
			CreateWorldFromSaveFile();
		}
		else {
			CreateEmptyWorld();
		}
	}

	void Update() {
		// TODO: Add pause/unpause, speed controls, etc...
		world.Update(Time.deltaTime);

	}
		
	/// <summary>
	/// Gets the tile at the unity-space coordinates
	/// </summary>
	/// <returns>The tile at world coordinate.</returns>
	/// <param name="coord">Unity World-Space coordinates.</param>
	public Tile GetTileAtWorldCoord(Vector3 coord) {
		int x = Mathf.FloorToInt(coord.x);
		int y = Mathf.FloorToInt(coord.y);
		
		return world.GetTileAt(x, y);
	}

	public void NewWorld() {
		Debug.Log("NewWorld button was clicked.");

		SceneManager.LoadScene( SceneManager.GetActiveScene().name );
	}

	public void SaveWorld() {
		Debug.Log("SaveWorld button was clicked.");

		XmlSerializer serializer = new XmlSerializer( typeof(World) );
		TextWriter writer = new StringWriter();
		serializer.Serialize(writer, world);
		writer.Close();

		Debug.Log( writer.ToString() );

		PlayerPrefs.SetString("SaveGame00", writer.ToString());

	}

	public void LoadWorld() {
		Debug.Log("LoadWorld button was clicked.");

		// Reload the scene to reset all data (and purge old references)
		loadWorld = true;
		SceneManager.LoadScene( SceneManager.GetActiveScene().name );

	}

	void CreateEmptyWorld() {
		// Create a world with Empty tiles
		world = new World(100, 100);

		// Center the Camera
		Camera.main.transform.position = new Vector3( world.Width/2, world.Height/2, Camera.main.transform.position.z );

	}

	void CreateWorldFromSaveFile() {
		Debug.Log("CreateWorldFromSaveFile");
		// Create a world from our save file data.

		XmlSerializer serializer = new XmlSerializer( typeof(World) );
		TextReader reader = new StringReader( PlayerPrefs.GetString("SaveGame00") );
		Debug.Log( reader.ToString() );
		world = (World)serializer.Deserialize(reader);
		reader.Close();



		// Center the Camera
		Camera.main.transform.position = new Vector3( world.Width/2, world.Height/2, Camera.main.transform.position.z );

	}

}
